"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var path = require("path");
var ComplieToolsFactory_1 = require("./ComplieToolsFactory");
var BaseTools_1 = require("./BaseTools");
var chokidar = require('chokidar');
var SbtModule_1 = require("../types/SbtModule");
var ts = require("typescript");
var TsSourceFile_1 = require("../types/TsSourceFile");
var ComplieTools = (function (_super) {
    __extends(ComplieTools, _super);
    function ComplieTools(moduleName, configFiles, tsComplieOptions, isDepModule) {
        var _this = _super.call(this, configFiles) || this;
        _this._watched = false;
        _this._needComplieFiles = [];
        _this.firstBuild = true;
        _this.firstBuildFinish = false;
        _this._forceMode = false;
        _this._buildChildMode = false;
        if (moduleName.indexOf('.ts') > -1) {
            _this.initFiles(moduleName);
        }
        else {
            _this.init(moduleName, configFiles, tsComplieOptions, isDepModule);
        }
        return _this;
    }
    ComplieTools.prototype.enableQuickMode = function () {
        this._quickMode = true;
    };
    ComplieTools.prototype.disableQuickMode = function () {
        this._quickMode = false;
    };
    ComplieTools.prototype.initFiles = function (files) {
        this._quickMode = true;
        this._needComplieFiles = files.split(' ');
        console.log(this._needComplieFiles);
    };
    ComplieTools.prototype.init = function (moduleName, configFiles, tsComplieOptions, isDepModule) {
        this._moduleName = moduleName;
        this._tsComplieOptions = tsComplieOptions;
        this._isCompling = false;
        this._isDepModule = isDepModule;
        this._factory = new ComplieToolsFactory_1.ComplieToolsFactory();
        this.sbtModule = new SbtModule_1.SbtModule(this.getModuleConfig(moduleName));
    };
    ComplieTools.prototype.getThisModuleConfig = function () {
        var moduleName = this.getModuleName();
        return this.getModuleConfig(moduleName);
    };
    ComplieTools.prototype.getModuleName = function () {
        return this._moduleName;
    };
    ComplieTools.prototype.getBuildDest = function () {
        return this._dest || path.join('build');
    };
    ComplieTools.prototype.setBuildDest = function (dest) {
        this._dest = dest;
    };
    ComplieTools.prototype.quickComplie = function () {
        this.watchFilesThenBuild();
    };
    ComplieTools.prototype.watchFilesThenBuild = function () {
        var _this = this;
        if (!this._watched) {
            this._watched = true;
            var dir = this.getModulePath(this.getModuleName());
            console.log('watching ' + dir);
            var build = function (msg, printMsg) {
                console.log(msg);
                _this.build();
            };
            var watcher = chokidar.watch(dir, { ignored: /^\./, persistent: true });
            this.sbtModule.isWatching = true;
            watcher
                .on('add', function (file) {
                _this.sbtModule.updateModifyTime();
                if (_this._forceMode) {
                    _this.addFileToNextComplie(file);
                }
                if (file.indexOf('_ref') > -1) {
                    _this.sbtModule.addRef(file);
                }
                if (!_this.firstBuild && _this.firstBuildFinish) {
                    build(file + ' added!');
                }
            })
                .on('change', function (file) {
                _this.sbtModule.updateModifyTime();
                _this.addFileToNextComplie(file);
                build(file + ' changed!');
            })
                .on('unlink', function (filepath) {
                _this.sbtModule.updateModifyTime();
                build(filepath + ' unlink!', true);
            })
                .on('error', function (error) { console.error('Error happened', error); })
                .on('ready', function () {
                build('watcher scan ready, start build ' + _this.getModuleName() + '...');
            });
        }
    };
    ComplieTools.prototype.addFileToNextComplie = function (file) {
        if (TsSourceFile_1.TsSourceFile.isTsSourceFile(file) && this._needComplieFiles.indexOf(file) == -1) {
            this._needComplieFiles.push(file);
        }
    };
    ComplieTools.prototype.clearNeedComplieFiles = function () {
        this._needComplieFiles = [];
    };
    ComplieTools.prototype.getNeedComplieFiles = function () {
        return this._needComplieFiles;
    };
    ComplieTools.prototype.getAllProjectFiles = function () {
        return this.sbtModule.getAllTsFiles();
    };
    ComplieTools.prototype.build = function (callback) {
        var _this = this;
        if (!this._tsComplieOptions) {
            this.prepareComplieOptions();
        }
        if (!this._isCompling && this.sbtModule.isDirty()) {
            this._isCompling = true;
            var build = function (callback) {
                if (!_this.firstBuild) {
                    _this.firstBuildFinish = true;
                    _this.firstBuild = false;
                }
                if (_this._buildChildMode) {
                    _this.buildDeps();
                }
                _this.complie();
                if (callback)
                    callback();
            };
            setTimeout(build, 10);
        }
    };
    ComplieTools.prototype.enableForceMode = function () {
        this._forceMode = true;
    };
    ComplieTools.prototype.prepareComplieOptions = function () {
        this._tsComplieOptions = {
            target: ts.ScriptTarget.ES5,
            noImplicitAny: false,
            removeComments: false,
            noLib: false,
            preserveConstEnums: true,
            suppressImplicitAnyIndexErrors: true,
            emitDecoratorMetadata: true,
            experimentalDecorators: true,
            sourceMap: true,
            jsx: ts.JsxEmit.React,
            noImplicitUseStrict: true,
            moduleResolution: ts.ModuleResolutionKind.NodeJs
        };
        var moduleConfig = this.getThisModuleConfig();
        if (moduleConfig.loaderType == 'amd') {
            this._tsComplieOptions.module = ts.ModuleKind.AMD;
            this._tsComplieOptions.outFile = './build/' + this.getModuleName() + '.js';
            if (this._quickMode) {
                console.log("can't use quick mode with amd module build yet!");
                process.exit();
            }
            if (this._buildChildMode) {
                console.log("you don't need use -c params with amd module build");
                process.exit();
            }
        }
        else {
            this._tsComplieOptions.outDir = './build';
        }
        if (moduleConfig.declaration) {
            this._tsComplieOptions.declaration = true;
        }
        if (moduleConfig.allowJs) {
            this._tsComplieOptions.allowJs = true;
        }
    };
    ComplieTools.prototype.complie = function () {
        console.log(new Date());
        var startBuildTime = new Date().getTime();
        if (!this._tsComplieOptions) {
            this.prepareComplieOptions();
        }
        this._isCompling = true;
        var files = [];
        if (this._quickMode) {
            files = this.getNeedComplieFiles();
        }
        else {
            files = this.getAllProjectFiles();
        }
        console.log('start build ' + this.getModuleName() + ', with ' + files.length + ' files... ');
        if (files.length > 0) {
            var refs = this.sbtModule.getRefs();
            for (var i in refs) {
                var ref = refs[i];
                if (files.indexOf(ref) == -1) {
                    files.push(ref);
                }
            }
        }
        if (this.isDebugModeEnabled()) {
            console.log(files.sort());
        }
        var program = ts.createProgram(files, this._tsComplieOptions || {
            noEmitOnError: true, noImplicitAny: true,
            target: ts.ScriptTarget.ES5, module: ts.ModuleKind.CommonJS,
            sourceMap: true, sourceRoot: ''
        });
        var emitResult = program.emit();
        var allDiagnostics = ts.getPreEmitDiagnostics(program).concat(emitResult.diagnostics);
        allDiagnostics.forEach(function (diagnostic) {
            var _a = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start), line = _a.line, character = _a.character;
            var message = ts.flattenDiagnosticMessageText(diagnostic.messageText, '\n');
            console.log(diagnostic.file.fileName + " (" + (line + 1) + "," + (character + 1) + "): " + message);
        });
        this.clearNeedComplieFiles();
        this._isCompling = false;
        var endBuildTime = new Date().getTime();
        console.log('build ' + this.getModuleName() + ' finish, time cost: ', endBuildTime - startBuildTime);
    };
    ComplieTools.prototype._buildInChildProcess = function (dep) {
    };
    ComplieTools.prototype.enableBuildChildMode = function () {
        this._buildChildMode = true;
    };
    ComplieTools.prototype.buildDeps = function () {
        var config = this.getModuleConfig(this.getModuleName());
        for (var i in config.deps) {
            var dep = config.deps[i];
            var buildTools = this._factory.createBuildTools(dep, this.getAllConfigFiles(), this._tsComplieOptions, true);
            buildTools.setBuildDest(this.getBuildDest());
            if (this._quickMode) {
                buildTools.enableQuickMode();
            }
            if (this.isDebugModeEnabled()) {
                buildTools.enableDebugMode();
            }
            if (this._forceMode) {
                buildTools.enableForceMode();
            }
            buildTools.enableBuildChildMode();
            if (this._quickMode) {
                buildTools.quickComplie();
            }
            else {
                buildTools.complie();
            }
        }
    };
    return ComplieTools;
}(BaseTools_1.BaseTools));
exports.ComplieTools = ComplieTools;
//# sourceMappingURL=ComplieTools.js.map