"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var ComplieTools_1 = require("./ComplieTools");
var fs = require("fs");
var path = require("path");
var fsExtra = require('fs-extra');
var UglifyJS = require("uglify-js");
var lzma = require('lzma');
var version_1 = require("../utils/version");
var PublishTools = (function (_super) {
    __extends(PublishTools, _super);
    function PublishTools() {
        return _super.apply(this, arguments) || this;
    }
    PublishTools.prototype.publish = function () {
        var _this = this;
        this.complie();
        this.concat();
        this.filter();
        this.minify(function () {
            _this.binaryCompress(function () {
                _this.copyResource();
                _this.addVersionNumber();
            });
        });
    };
    PublishTools.prototype.getPublishConfig = function () {
        var config = this.getModuleConfig(this.getModuleName());
        return config.publish;
    };
    PublishTools.prototype.concat = function () {
        var config = this.getPublishConfig();
        var enabled = config.concat === false ? false : true;
        if (enabled) {
            if (config.beforeConcat instanceof Function) {
                config.beforeConcat();
            }
            var concatFiles = config.concatFiles;
            var allStr = '';
            for (var i in concatFiles) {
                var file = concatFiles[i];
                var content = this.readFile(file);
                allStr += content;
                allStr += '\n';
            }
            this.writeFile(this.getConcatedFile(), allStr);
            if (config.afterConcat instanceof Function) {
                config.afterConcat();
            }
        }
    };
    PublishTools.prototype.getConcatedFile = function () {
        return path.join('build', this.getModuleName() + '.tmp.js');
    };
    PublishTools.prototype.getFiltedFile = function () {
        return path.join('build', this.getModuleName() + '.tmp.js');
    };
    PublishTools.prototype.getMinifyedFile = function () {
        return path.join('build', this.getModuleName() + '.tmp.js');
    };
    PublishTools.prototype.getBinaryedFile = function () {
        return path.join('build', this.getModuleName() + '.jr');
    };
    PublishTools.prototype.filter = function () {
        var config = this.getPublishConfig();
        var enabled = config.filter === false ? false : true;
        if (enabled && config.filterFunction) {
            if (config.beforeFliter instanceof Function) {
                config.beforeFliter();
            }
            var content = this.readFile(this.getConcatedFile());
            var lines = content.split('\n');
            var newContent = '';
            for (var i = 0; i < lines.length; i++) {
                var line = lines[i];
                if (i != lines.length - 1) {
                    newContent += config.filterFunction(line) + '\n';
                }
                else {
                    newContent += config.filterFunction(line);
                }
            }
            this.writeFile(this.getFiltedFile(), newContent);
            if (config.afterFliter instanceof Function) {
                config.afterFliter();
            }
        }
    };
    PublishTools.prototype.minify = function (cb) {
        var config = this.getPublishConfig();
        var enabled = config.minify === false ? false : true;
        if (enabled) {
            if (config.beforeMinify instanceof Function) {
                config.beforeMinify();
            }
            var result = UglifyJS.minify([this.getFiltedFile()], {
                compress: {
                    dead_code: true,
                    global_defs: {
                        DEBUG: false
                    }
                },
                mangle: { toplevel: true },
                mangleProperties: false,
                output: {
                    ascii_only: true
                }
            });
            this.writeFile(this.getMinifyedFile(), result.code);
            if (config.afterMinify instanceof Function) {
                config.afterMinify();
            }
            cb();
        }
    };
    PublishTools.prototype.binaryCompress = function (cb) {
        var config = this.getPublishConfig();
        var enabled = config.binaryCompress === false ? false : true;
        var binaryedFile = this.getBinaryedFile();
        if (enabled) {
            if (config.beforeBinrayCompress) {
                config.beforeBinrayCompress();
            }
            function onCompressProgress(percent) {
                console.log('percent: ' + Math.round(percent * 10000) / 100 + '%');
            }
            function onCompressFinish(result, error) {
                fs.writeFileSync(binaryedFile, new Buffer(result), { encoding: 'hex' });
                console.log('Write output file. (file=' + binaryedFile + ', length=' + result.length + ')');
                if (config.afterBinrayCompress) {
                    config.afterBinrayCompress();
                }
                cb();
            }
            var code = fs.readFileSync(this.getMinifyedFile());
            lzma.compress(code, 1, onCompressFinish, onCompressProgress);
        }
    };
    PublishTools.prototype.copyResource = function () {
        var config = this.getPublishConfig();
        var enabled = config.copyResource === false ? false : true;
        if (enabled && config.copyResourceActions) {
            if (config.beforeCopyResource instanceof Function) {
                config.beforeCopyResource();
            }
            for (var i in config.copyResourceActions) {
                var action = config.copyResourceActions[i];
                fsExtra.copySync(action.from, action.to);
            }
            if (config.afterCompyResource instanceof Function) {
                config.afterCompyResource();
            }
        }
    };
    PublishTools.prototype.addVersionNumber = function () {
        var config = this.getPublishConfig();
        var enabled = config.addVersionNumber === false ? false : true;
        if (enabled && config.needVersionFiles) {
            if (config.beforeAddVersionNumber instanceof Function) {
                config.beforeAddVersionNumber();
            }
            version_1.addVersionNumberToFiles(config.needVersionFiles);
            if (config.afterAddVersionNumber instanceof Function) {
                config.afterAddVersionNumber();
            }
        }
    };
    PublishTools.prototype.readFile = function (file) {
        return fs.readFileSync(file, 'utf-8');
    };
    PublishTools.prototype.writeFile = function (file, content) {
        fs.writeFileSync(file, content, 'utf-8');
    };
    return PublishTools;
}(ComplieTools_1.ComplieTools));
exports.PublishTools = PublishTools;
//# sourceMappingURL=PublishTools.js.map