#!/usr/bin/env node
"use strict";
var program = require("commander");
var BuildTools_1 = require("./lib/tools/BuildTools");
var PublishTools_1 = require("./lib/tools/PublishTools");
var ComplieToNativeTools_1 = require("./lib/tools/ComplieToNativeTools");
program.version('1.08.9');
program.command('build <module_name>')
    .option("-q, --quickMode")
    .option("-f, --forceMode")
    .option("-d, --debugMode")
    .option("-c, --buildChildMode")
    .option("-n, --nativeMode")
    .option("-w, --watchMode")
    .option("-l, --livereload")
    .action(function (moduleName, options) {
    console.log('build ' + moduleName);
    if (options.nativeMode) {
        var buildTools = new ComplieToNativeTools_1.ComplieToNativeTools(moduleName);
        buildTools.complie();
    }
    else {
        var buildTools = new BuildTools_1.BuildTools(moduleName);
        if (moduleName.indexOf('.ts') > -1) {
            buildTools.build();
        }
        else {
            if (options.quickMode) {
                buildTools.enableQuickMode();
            }
            if (options.debugMode) {
                buildTools.enableDebugMode();
            }
            if (options.forceMode) {
                buildTools.enableForceMode();
            }
            if (options.buildChildMode) {
                buildTools.enableBuildChildMode();
            }
            if (options.watchMode) {
                buildTools.enableWatchMode();
            }
            if (options.quickMode) {
                buildTools.quickComplie();
            }
            else {
                buildTools.build();
            }
        }
    }
    if (options.livereload) {
        startLivereloadServer();
    }
});
function startLivereloadServer() {
    var livereload = require('livereload');
    var server = livereload.createServer();
    server.watch("./build");
}
program.command('publish <module_name>')
    .option("-d, --debugMode")
    .action(function (moduleName, options) {
    console.log('build ' + moduleName);
    var publishTools = new PublishTools_1.PublishTools(moduleName);
    if (options.debugMode) {
        publishTools.enableDebugMode();
    }
    publishTools.publish();
});
program.on('--help', function () {
    console.log('  Examples:');
    console.log();
    console.log('    $ sbt publish src/tools');
    console.log('    $ sbt build src/tools -w');
    console.log('    $ sbt init src/plugin/myplugin');
    console.log();
});
program.parse(process.argv);
//# sourceMappingURL=main.js.map